<?php
	set_time_limit(900);
	require_once "DB.class.php";
	require 'vendor/autoload.php';

	use GeoIp2\Database\Reader;
	$reader = new Reader('GeoLite2-Country.mmdb');
	
	/* === functions === */
	/*function log($message) {
		
	}*/
	function getDatabaseNames() {
		$sql = "SELECT SCHEMA_NAME AS `Database` FROM INFORMATION_SCHEMA.SCHEMATA WHERE SCHEMA_NAME LIKE 'usr_bn%'";
		$query = DB::getDB()->prepare($sql);
		$query->execute();
		$query->setFetchMode(PDO::FETCH_COLUMN, 0);
		return $query->fetchAll();
	}
	
	function fetchData($dbname) {
		$sql = "SELECT max(tstamp), min(tstamp), IP, userid FROM `$dbname`.`sys_log` WHERE userid != 0 AND userid NOT IN (SELECT uid FROM `$dbname`.`be_users` WHERE username = '_cli_lowlevel' OR username = '_cli_scheduler' OR username = '_cli_') GROUP BY userid, IP";
		$query = DB::getDB()->prepare($sql);
		try {
			$query->execute();
			return $query->fetchAll(PDO::FETCH_ASSOC);
		} catch (PDOException $ex) {
			echo "<p>Error with $dbname : ".$ex->getMessage().'</p>';
			return [];
		}
 		return false;
	}

	function printTable($data) {
		echo '<table border="1">';
		echo '<thead>';
		echo '<tr>';
		foreach($data[0] as $colname => $coldata) {
			echo '<th>'. $colname .'</th>';
		}
		echo '</tr>';
		echo '</thead>';
		echo '<tbody>';
		foreach($data as $row) {
			echo '<tr>';
			foreach($row as $colname => $coldata) {
				echo '<td>'. $coldata .'</td>';
			}
			echo '</tr>';
		}
		echo '</tbody>';
		echo '</table>';
	}

	function getDomainFromDB($database) {
		// No '_'? Stop.
		if (strpos($database, '_') === false)
			return false;

		$user = explode('_', $database)[1];

		// Contains ' '? Stop.
		if (strpos($user, ' ') !== false)
			return false;
		
		$home = exec('eval echo ~'.escapeshellarg($user));

		// No '/'? Stop.
		if (strpos($home, '/') === false)
			return false;

		$domain = explode('/', $home)[4];

		return $domain;
		
	}

	function countryCodeToEmoji($cc) {
		return 
			mb_chr(127397 + ord($cc[0])) .
			mb_chr(127397 + ord($cc[1]));
	}

	/* === main === */

	// Get databases
	$databases = getDatabaseNames();

	// HTML Pre-Span
?>
<!DOCTYPE html>
<html lang=en>
	<head>
		<title>Database Scan Report</title>
		<meta charset=UTF-8 />
	</head>
	<body>
		<p>Filter: <a href=?>on</a>/<a href=?nofilter>off</a></p>
<?php
	// Fetch data and print
	foreach($databases as $database) {
		echo '<h1>'. $database .'</h1>';
		echo '<h2>'. getDomainFromDB($database) .'</h2>';
		$data = fetchData($database);
		$refined_data = [];
		foreach ($data as $value) {
			try {
				if(!isset($_GET['nofilter']) && !filter_var($value['IP'], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE))
					continue;
				$record = $reader->country($value['IP']);
				if(!isset($_GET['nofilter']) && in_array($record->country->isoCode, ['IT', 'DE', 'AT', 'CH']))
					continue;
				$refined_data[] = [
					'Timestamp min' => date_format(date_create('@'. $value['max(tstamp)']), 'Y-m-d H:i:s'),
					'Timestamp max' => date_format(date_create('@'. $value['min(tstamp)']), 'Y-m-d H:i:s'),
					'UserID' => $value['userid'],
					'IP' => '<a target=_blank href=https://ip-api.com/#'.$value['IP'].'>'.$value['IP'].'</a>',
					'Country' => @countryCodeToEmoji($record->country->isoCode) . '&nbsp;' .  @$record->country->names['de']
				];
			} catch (\Exception $ex) {
				$refined_data[] = [
					'Timestamp min' => date_format(date_create('@'. $value['max(tstamp)']), 'Y-m-d H:i:s'),
					'Timestamp max' => date_format(date_create('@'. $value['min(tstamp)']), 'Y-m-d H:i:s'),
					'UserID' => $value['userid'],
					'IP' => $value['IP'],
					'Country' => '✨'
				];
			}
		}
		if(count($refined_data) > 0)
			printTable($refined_data);
		else
			echo '<p><i>empty</i></p>';
	}
	
?>
	</body>
</html>